#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs-extra");
var child_process = require("child_process");
var glob = require("glob");
var path = require("path");
var yaml = require("js-yaml");
var program = require("commander");
var src;
var repoRelativePath;
var dest = 'docs-ref-autogen';
var packageWhiteList = [];
var packageBlackList = [];
var packageOnlyRootList = [];
var config;
var pjson = require('../package.json');
var envPath = process.env['APPDATA'] + "/npm/node_modules/ts-reference-ci-scripts";
program
    .version("v" + pjson.version)
    .description('A tool to go through source code and convert it to yaml data model for docfx.')
    .option('--config [value]', 'The config file which will defined more specefied behaviour for the tool')
    .option('--destPath [value]', 'Path to output the autogen files.')
    .arguments('<sourceFolder> [repoConfigPath]')
    .action(function (sourceFolder, repoConfig) {
    src = sourceFolder;
    repoRelativePath = repoConfig;
})
    .parse(process.argv);
if (!src) {
    console.log('Error: The source file folder is not specified!');
    program.help();
}
if (program.config) {
    try {
        config = JSON.parse(program.config);
    }
    catch (e) {
        console.log("Invalid json format of config, ignored! content: " + program.config);
    }
}
if (config) {
    if (config.package_white_list) {
        packageWhiteList = config.package_white_list;
    }
    if (config.package_black_list) {
        packageBlackList = config.package_black_list;
    }
    if (config.npm_package_sources && config.npm_package_sources.length) {
        config.npm_package_sources.forEach(function (npmPackageSource) {
            var npmPackageSourceName = npmPackageSource.name.trim();
            child_process.execSync("npm install " + npmPackageSourceName + " 2>&1");
            fs.copySync("node_modules/" + npmPackageSourceName, src + "/" + npmPackageSourceName);
        });
    }
    if (config.package_only_root) {
        packageOnlyRootList = config.package_only_root;
    }
}
if (program.destPath) {
    if (config.target_repo.folder) {
        dest = program.destPath + "/" + config.target_repo.folder + "/" + dest;
    }
    else {
        dest = program.destPath + "/" + dest;
    }
}
// 1. prepare
fs.removeSync(dest);
fs.mkdirpSync(dest);
var packagePaths = glob.sync(path.join(src, '**/package.json'));
var toc = [];
var jsdocConfig = require('../config/node2docfx.json');
packagePaths.forEach(function (packagePath) {
    var dir = path.dirname(packagePath);
    if (!fs.existsSync(packagePath)) {
        return;
    }
    var packageName = fs.readJsonSync(packagePath).name;
    if (packageWhiteList.length === 0 || packageWhiteList.indexOf(packageName) >= 0) {
        if (packageBlackList.indexOf(packageName) >= 0) {
            return;
        }
        generatePackageDoc(packagePath, packageName, dest, jsdocConfig, config);
    }
});
toc = JSON.parse(JSON.stringify(toc));
fs.writeFileSync(dest + "/toc.yml", yaml.safeDump(toc));
process.exit(0);
function generatePackageDoc(packagePath, packageName, dest, jsdocConfig, config) {
    console.log(packageName + " : " + packagePath);
    var dir = path.dirname(packagePath);
    var folder = dir.split('/')[1];
    var sourcePath = dir;
    if (fs.existsSync(dir + "/lib")) {
        sourcePath = dir + "/lib";
    }
    jsdocConfig.source.include = sourcePath;
    if (config.source_repos && config.source_repos.length) {
        var folderIndex = parseInt(folder, 10);
        jsdocConfig.repo = [config.source_repos[folderIndex]];
    }
    jsdocConfig.package = packagePath;
    jsdocConfig.readme = path.join(dir, 'README.md');
    jsdocConfig.destination = path.join(dest, packageName);
    if (packageOnlyRootList.indexOf(packageName) >= 0) {
        jsdocConfig.recurseDepth = 1;
    }
    else {
        jsdocConfig.recurseDepth = 10;
    }
    fs.writeJsonSync('_node2docfx_temp.json', jsdocConfig);
    try {
        child_process.execSync('node2docfx ' + '_node2docfx_temp.json 2>&1');
    }
    catch (e) { }
    var outputs = fs.readdirSync(jsdocConfig.destination);
    if (outputs.length === 1) {
        console.log('No YAML files genereated for ' + packageName);
        fs.removeSync(jsdocConfig.destination);
    }
    else {
        glob.sync(path.join(jsdocConfig.destination, '**/*..yml')).forEach(function (unwantedFile) {
            console.log("DELETE " + unwantedFile);
            fs.unlinkSync(unwantedFile);
        });
        if (fs.existsSync(jsdocConfig.destination + "/toc.yml")) {
            var subToc = yaml.safeLoad(fs.readFileSync(jsdocConfig.destination + "/toc.yml").toString());
            var tocContent = { name: packageName, uid: packageName, landingPageType: 'Service', items: subToc };
            toc.push(tocContent);
            fs.removeSync(jsdocConfig.destination + "/toc.yml");
        }
        console.log('Finish generating YAML files for ' + packageName);
    }
}
